from __future__ import print_function
import os
import sys

pwd = os.getcwd()

if 'opt' in pwd:
    os.environ['SUMO_HOME'] = '/sumo'
else:
    sh_path = os.path.expanduser('~/sumo')
    os.environ['SUMO_HOME'] = sh_path

sbp = '/sumo/bin'
os.environ["PATH"] += os.pathsep + sbp

if 'SUMO_HOME' in os.environ:
    tools = os.path.join(os.environ['SUMO_HOME'], 'tools')
    sys.path.append(tools)
else:
    sys.exit("please declare environment variable 'SUMO_HOME'")

import traci
import traci.constants as tc
import numpy as np

from sumolib import checkBinary


sys.path.append('../src')


class SumoEnv:
    def __init__(self, cfg_file, show_gui=False, sumo_step_length=0.5, lane_change_duration=2.0, sumo_path="", scenario_path=None):

        self.sumo_step_length = sumo_step_length
        self.lane_change_duration = lane_change_duration
        self.cfg_file = cfg_file
        self.show_gui = show_gui
        self.time_step = 0
        self.scenario_path = scenario_path
        self.sumo_path = sumo_path
        self.start_sumo()
        self.max_view = 80.0

        self.filter_lanes = [-2, -1, 0, 1, 2]
        self.departLane = "1"
        self.add_routes_straight()

        self.edges = ["gneE0"]

    def add_routes_straight(self):
        self.conn.route.add("trip_1", ["gneE0"])

    def start_sumo(self):
        if not self.show_gui:
            sumo_binary = checkBinary('sumo', self.sumo_path)
        else:
            sumo_binary = checkBinary('sumo-gui', self.sumo_path)
        seed = np.random.randint(10000)
        traci.start([sumo_binary, "-c", self.cfg_file,
                     "--lanechange.duration", str(self.lane_change_duration),
                     "--step-length", str(self.sumo_step_length),
                     "--lanechange.overtake-right", "False",
                     "--no-internal-links", "False",
                     "--collision.action", "none",
                     "--no-step-log", "True"], label="sim%d" % seed)
        self.conn = traci.getConnection("sim%d" % seed)

    def reset_sumo(self, scenario_id=None, disable_sumo_lanechanges=True):
        print("------------------------------------   RESETTING SUMO   ------------------------------------")

        self.time_step = 0
        try:
            self.conn.vehicle.unsubscribeContext("EGO", tc.CMD_GET_VEHICLE_VARIABLE, 1000)
            print('Ego unsubscribed')
        except:
            print('No subscriptions')

        # remove vehicle
        for veh_id in self.conn.vehicle.getIDList():
            try:
                self.conn.vehicle.remove(veh_id)
            except:
                print("Exception:  remove vehicle ", veh_id)
        try:
            self.conn.simulationStep()
        except:
            self.reset()
        print('-------------------------------------------------  ADDING EGO VEHICLE TO THE SIMULATION -------------------------------------------------')
        self.conn.vehicle.addFull("EGO", routeID="trip_1", typeID="ego_car", departLane=self.departLane, departPos="100.", departSpeed="8.")
        self.conn.vehicle.setSpeedMode("EGO", 0)
        self.conn.vehicle.setColor("EGO", (255, 165, 0, 200))
        if self.show_gui:
            default_view = 'View #0'
            self.conn.gui.trackVehicle(default_view, 'EGO')
            self.conn.gui.setZoom(default_view, 250.)
        if disable_sumo_lanechanges:
            self.conn.vehicle.setLaneChangeMode("EGO", 512)

        self.conn.vehicle.subscribeContext("EGO", tc.CMD_GET_VEHICLE_VARIABLE, 0.0,
                                           [tc.VAR_POSITION, tc.VAR_LANE_INDEX, tc.VAR_SPEED, tc.VAR_ACCEL,
                                            tc.VAR_ALLOWED_SPEED, tc.VAR_LANEPOSITION, tc.VAR_LANEPOSITION_LAT,
                                            tc.VAR_LANE_ID, tc.VAR_LENGTH, tc.VAR_WIDTH, tc.VAR_ROUTE_ID])
        self.conn.vehicle.addSubscriptionFilterLanes(self.filter_lanes, downstreamDist=self.max_view, upstreamDist=self.max_view)

        print('Scenario ID: ', scenario_id)
        # Adding the surrounding vehicles in the simulation

        car_list = np.load(os.path.join(self.scenario_path, "%s_vTypes.npy" % scenario_id))
        car_pos = np.load(os.path.join(self.scenario_path, "%s_vPos.npy" % scenario_id))

        for i, type_id in enumerate(car_list):
            self.conn.vehicle.addFull("HUMAN_%d" % i, routeID=car_pos[i][0], typeID=type_id, departLane=car_pos[i][1], departPos=car_pos[i][2], departSpeed="10.")
            self.conn.vehicle.setColor("HUMAN_%d" % i, (0, 255, 0, 255))

        self.conn.simulationStep()

    def reset(self, scenario_id=None):
        self.reset_sumo(scenario_id)

    def execute(self, pos_x, pos_y, angle, speed):

        edge_id = self.conn.vehicle.getRoadID(vehID='EGO')
        lane_index = self.conn.vehicle.getLaneIndex(vehID='EGO')

        #self.conn.vehicle.moveToXY(vehID='EGO', edgeID=edge_id, lane=lane_index, x=pos_x, y=pos_y, angle=angle, keepRoute=2)
        self.conn.vehicle.moveToXY(vehID='EGO', edgeID="", lane=lane_index, x=pos_x, y=pos_y, angle=angle, keepRoute=7)
        self.conn.vehicle.setSpeed(vehID='EGO', speed=speed)
        self.conn.simulationStep()